import urllib.parse
import transmission_rpc
from fastapi import Depends, FastAPI, Request
from fastapi.responses import HTMLResponse
from fastapi.templating import Jinja2Templates

from torrent_downloader.client import TorrentDownloader

app = FastAPI()
templates = Jinja2Templates(directory="templates")


def get_downloader() -> TorrentDownloader:
    return TorrentDownloader()


def get_active_torrents(downloader=get_downloader()) -> list[transmission_rpc.Torrent]:
    return [t for t in downloader.get_active_torrents() if t.format_eta() != "not available"]


@app.get("/", response_class=HTMLResponse)
async def index(request: Request):
    return templates.TemplateResponse(request=request, name="index.html")


@app.get("/list", response_class=HTMLResponse)
async def list_torrents(request: Request, query: str, downloader=Depends(get_downloader)):
    torrents = downloader.search(query)
    return templates.TemplateResponse(
        request=request, name="torrents.html", context={"torrents": torrents}
    )


@app.get("/download", response_class=HTMLResponse)
async def download(request: Request, downloader=Depends(get_downloader)):
    magnet = urllib.parse.unquote(str(request.query_params))
    downloader.download_magnet(magnet)
    active = downloader.get_active_torrents()
    return templates.TemplateResponse(
        request=request, name="active_torrents.html", context={"torrents": active}
    )


@app.get("/active", response_class=HTMLResponse)
async def active(request: Request, active=Depends(get_active_torrents)):
    return templates.TemplateResponse(
        request=request, name="active_torrents.html", context={"torrents": active}
    )
